@@ -1,5 +1,5 @@
 /* ldapmodify.c - generic program to modify or add entries using LDAP */
-/* $OpenLDAP: pkg/ldap/clients/tools/ldapmodify.c,v 1.158.2.10 2007/01/02 21:43:41 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/clients/tools/ldapmodify.c,v 1.158.2.13 2007/08/13 20:03:51 ando Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 1998-2007 The OpenLDAP Foundation.
@@ -389,8 +389,8 @@ main( int argc, char **argv )
 			fprintf( rejfp, "\n%s\n", rejbuf );
 		}
 
-		if (rejfp) free( rejbuf );
-		free( rbuf );
+		if (rejfp) ber_memfree( rejbuf );
+		ber_memfree( rbuf );
 	}
 
 #ifdef LDAP_GROUP_TRANSACTION
@@ -519,7 +519,7 @@ process_ldif_rec( char *rbuf, int count )
 				printf(_("%s: skipping change record for entry: %s\n"),
 					prog, dn);
 				printf(_("\t(LDAP host/port does not match replica: lines)\n"));
-				free( dn );
+				ber_memfree( dn );
 				ber_memfree( type );
 				ber_memfree( val.bv_val );
 				return( 0 );
@@ -727,13 +727,13 @@ end_line:
 	}
 
 	if ( dn != NULL ) {
-		free( dn );
+		ber_memfree( dn );
 	}
 	if ( newrdn != NULL ) {
-		free( newrdn );
+		ber_memfree( newrdn );
 	}
 	if ( newsup != NULL ) {
-		free( newsup );
+		ber_memfree( newsup );
 	}
 	if ( pmods != NULL ) {
 		ldap_mods_free( pmods, 1 );
@@ -1155,6 +1155,7 @@ static int process_response(
 
 		if ( rc == -1 ) {
 			ldap_get_option( ld, LDAP_OPT_ERROR_NUMBER, &rc );
+			tool_perror( "ldap_result", rc, NULL, NULL, NULL, NULL );
 			return rc;
 		}
 
@@ -1164,9 +1165,51 @@ static int process_response(
 	}
 
 	if ( ldap_msgtype( res ) != LDAP_RES_INTERMEDIATE ) {
-		rc = ldap_result2error( ld, res, 1 );
-		if( rc != LDAP_SUCCESS ) ldap_perror( ld, opstr );
-		return rc;
+		int code;
+		char *matcheddn = NULL, *text = NULL, **refs = NULL;
+		LDAPControl **ctrls = NULL;
+		rc = ldap_parse_result( ld, res, &code, &matcheddn, &text, &refs, &ctrls, 1 );
+
+		if ( rc != LDAP_SUCCESS ) {
+			fprintf( stderr, "%s: ldap_parse_result: %s (%d)\n",
+				prog, ldap_err2string( rc ), rc );
+			return rc;
+		}
+
+		if ( code != LDAP_SUCCESS ) {
+			tool_perror( prog, code, NULL, matcheddn, text, refs );
+		} else if ( verbose && 
+			((matcheddn && *matcheddn) || (text && *text) || (refs && *refs) ))
+		{
+			printf( _("Delete Result: %s (%d)\n"),
+				ldap_err2string( code ), code );
+
+			if ( text && *text ) {
+				printf( _("Additional info: %s\n"), text );
+			}
+
+			if ( matcheddn && *matcheddn ) {
+				printf( _("Matched DN: %s\n"), matcheddn );
+			}
+
+			if ( refs ) {
+				int i;
+				for( i=0; refs[i]; i++ ) {
+					printf(_("Referral: %s\n"), refs[i] );
+				}
+			}
+		}
+
+		if (ctrls) {
+			tool_print_ctrls( ld, ctrls );
+			ldap_controls_free( ctrls );
+		}
+
+		ber_memfree( text );
+		ber_memfree( matcheddn );
+		ber_memvfree( (void **) refs );
+
+		return code;
 	}
 
 #ifdef LDAP_GROUP_TRANSACTION
