@@ -1,8 +1,17 @@
 /* init.c - initialize ldbm backend */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-ldbm/init.c,v 1.76.2.6 2003/02/09 16:31:38 kurt Exp $ */
-/*
- * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-ldbm/init.c,v 1.96.2.8 2007/01/02 21:44:02 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 1998-2007 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
  */
 
 #include "portable.h"
@@ -14,21 +23,7 @@
 
 #include "slap.h"
 #include "back-ldbm.h"
-
-#ifdef SLAPD_LDBM_DYNAMIC
-
-int init_module(int argc, char *argv[]) {
-    BackendInfo bi;
-
-    memset( &bi, '\0', sizeof(bi) );
-    bi.bi_type = "ldbm";
-    bi.bi_init = ldbm_back_initialize;
-
-    backend_add(&bi);
-    return 0;
-}
-
-#endif /* SLAPD_LDBM_DYNAMIC */
+#include <ldap_rq.h>
 
 int
 ldbm_back_initialize(
@@ -37,12 +32,20 @@ ldbm_back_initialize(
 {
 	static char *controls[] = {
 		LDAP_CONTROL_MANAGEDSAIT,
-		LDAP_CONTROL_VALUESRETURNFILTER,
+		LDAP_CONTROL_X_PERMISSIVE_MODIFY,
 		NULL
 	};
 
 	bi->bi_controls = controls;
 
+	bi->bi_flags |= 
+		SLAP_BFLAG_INCREMENT |
+#ifdef LDBM_SUBENTRIES
+		SLAP_BFLAG_SUBENTRIES |
+#endif
+		SLAP_BFLAG_ALIASES |
+		SLAP_BFLAG_REFERRALS;
+
 	bi->bi_open = ldbm_back_open;
 	bi->bi_config = NULL;
 	bi->bi_close = ldbm_back_close;
@@ -67,8 +70,7 @@ ldbm_back_initialize(
 	bi->bi_extended = ldbm_back_extended;
 
 	bi->bi_entry_release_rw = ldbm_back_entry_release_rw;
-	bi->bi_acl_group = ldbm_back_group;
-	bi->bi_acl_attribute = ldbm_back_attribute;
+	bi->bi_entry_get_rw = ldbm_back_entry_get;
 	bi->bi_chk_referrals = ldbm_back_referrals;
 	bi->bi_operational = ldbm_back_operational;
 	bi->bi_has_subordinates = ldbm_back_hasSubordinates;
@@ -85,6 +87,10 @@ ldbm_back_initialize(
 	bi->bi_tool_entry_reindex = ldbm_tool_entry_reindex;
 	bi->bi_tool_sync = ldbm_tool_sync;
 
+	bi->bi_tool_dn2id_get = 0;
+	bi->bi_tool_id2entry_get = 0;
+	bi->bi_tool_entry_modify = 0;
+
 	bi->bi_connection_init = 0;
 	bi->bi_connection_destroy = 0;
 
@@ -128,11 +134,6 @@ ldbm_back_db_init(
 {
 	struct ldbminfo	*li;
 
-	/* indicate system schema supported */
-	be->be_flags |= 
-		SLAP_BFLAG_ALIASES |
-		SLAP_BFLAG_REFERRALS;
-
 	/* allocate backend-database-specific stuff */
 	li = (struct ldbminfo *) ch_calloc( 1, sizeof(struct ldbminfo) );
 
@@ -172,8 +173,8 @@ ldbm_back_db_init(
 	/* delay interval */
 	li->li_dbsyncwaitinterval = 5;
 
-	/* flag to notify ldbm_cache_sync_daemon to shut down */
-	li->li_dbshutdown = 0;
+	/* current wait counter */
+	li->li_dbsyncwaitcount = 0;
 
 	/* initialize various mutex locks & condition variables */
 	ldap_pvt_thread_rdwr_init( &li->li_giant_rwlock );
@@ -192,27 +193,41 @@ ldbm_back_db_open(
 )
 {
 	struct ldbminfo *li = (struct ldbminfo *) be->be_private;
+	int rc;
+
+	rc = alock_open( &li->li_alock_info, "slapd",
+		li->li_directory, ALOCK_UNIQUE );
+	if ( rc == ALOCK_BUSY ) {
+		Debug( LDAP_DEBUG_ANY,
+			"ldbm_back_db_open: database already in use\n",
+			0, 0, 0 );
+		return -1;
+	} else if ( rc == ALOCK_RECOVER ) {
+		Debug( LDAP_DEBUG_ANY,
+			"ldbm_back_db_open: unclean shutdown detected;"
+			" database may be inconsistent!\n",
+			0, 0, 0 );
+		rc = alock_recover( &li->li_alock_info );
+	}
+	if ( rc != ALOCK_CLEAN ) {
+		Debug( LDAP_DEBUG_ANY,
+			"ldbm_back_db_open: alock package is unstable;"
+			" database may be inconsistent!\n",
+			0, 0, 0 );
+	}
 	li->li_dbenv = ldbm_initialize_env( li->li_directory,
 		li->li_dbcachesize, &li->li_envdirok );
 
-	/* sync thread */
-	if ( li->li_dbsyncfreq > 0 )
+	/* If we're in server mode and a sync frequency was set,
+	 * submit a task to perform periodic db syncs.
+	 */
+	if (( slapMode & SLAP_SERVER_MODE ) && li->li_dbsyncfreq > 0 )
 	{
-		int rc;
-		rc = ldap_pvt_thread_create( &li->li_dbsynctid,
-			0, ldbm_cache_sync_daemon, (void*)be );
-
-		if ( rc != 0 )
-		{
-#ifdef NEW_LOGGING
-			LDAP_LOG ( BACK_LDBM, ERR, "ldbm_back_db_open: sync "
-				"ldap_pvt_thread_create failed (%d)\n", rc, 0, 0 );
-#else	
-			Debug(	LDAP_DEBUG_ANY,
-				"sync ldap_pvt_thread_create failed (%d)\n", rc, 0, 0 );
-#endif
-			return 1;
-		}
+		ldap_pvt_thread_mutex_lock( &slapd_rq.rq_mutex );
+		ldap_pvt_runqueue_insert( &slapd_rq, li->li_dbsyncfreq,
+			ldbm_cache_sync_daemon, be,
+			"ldbm_cache_sync", be->be_suffix[0].bv_val );
+		ldap_pvt_thread_mutex_unlock( &slapd_rq.rq_mutex );
 	}
 
 	return 0;
@@ -242,3 +257,12 @@ ldbm_back_db_destroy(
 
 	return 0;
 }
+
+#if SLAPD_LDBM == SLAPD_MOD_DYNAMIC
+
+/* conditionally define the init_module() function */
+SLAP_BACKEND_INIT_MODULE( ldbm )
+
+#endif /* SLAPD_LDBM == SLAPD_MOD_DYNAMIC */
+
+
