@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-sql/entry-id.c,v 1.46.2.11 2006/08/17 17:53:17 ando Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-sql/entry-id.c,v 1.46.2.13 2007/08/11 07:39:30 ando Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1999-2006 The OpenLDAP Foundation.
+ * Copyright 1999-2007 The OpenLDAP Foundation.
  * Portions Copyright 1999 Dmitry Kovalev.
  * Portions Copyright 2002 Pierangelo Masarati.
  * Portions Copyright 2004 Mark Adamson.
@@ -1002,12 +1002,14 @@ next:;
 			|| an_find( bsi->bsi_attrs, &AllOper )
 			|| an_find( bsi->bsi_attrs, &slap_schema.si_ad_structuralObjectClass->ad_cname ) )
 	{
+		ObjectClass	*soc = NULL;
+
 		if ( BACKSQL_CHECK_SCHEMA( bi ) ) {
 			Attribute	*a;
 			const char	*text = NULL;
 			char		textbuf[ 1024 ];
 			size_t		textlen = sizeof( textbuf );
-			struct berval	soc,
+			struct berval	soc_name,
 					bv[ 2 ],
 					*nvals;
 			int		rc = LDAP_SUCCESS;
@@ -1023,7 +1025,7 @@ next:;
 				nvals = bv;
 			}
 
-			rc = structural_class( nvals, &soc, NULL, 
+			rc = structural_class( nvals, &soc_name, &soc, 
 					&text, textbuf, textlen );
 			if ( rc != LDAP_SUCCESS ) {
       				Debug( LDAP_DEBUG_TRACE, "backsql_id2entry(%s): "
@@ -1034,21 +1036,33 @@ next:;
 				return rc;
 			}
 
-			if ( !bvmatch( &soc, &bsi->bsi_oc->bom_oc->soc_cname ) ) {
+			if ( !bvmatch( &soc->soc_cname, &bsi->bsi_oc->bom_oc->soc_cname ) ) {
+				if ( !is_object_subclass( bsi->bsi_oc->bom_oc, soc ) ) {
+      					Debug( LDAP_DEBUG_TRACE, "backsql_id2entry(%s): "
+						"computed structuralObjectClass %s "
+						"does not match objectClass %s associated "
+						"to entry\n",
+						bsi->bsi_e->e_name.bv_val, soc->soc_cname.bv_val,
+						bsi->bsi_oc->bom_oc->soc_cname.bv_val );
+					backsql_entry_clean( op, bsi->bsi_e );
+					return rc;
+				}
+
       				Debug( LDAP_DEBUG_TRACE, "backsql_id2entry(%s): "
 					"computed structuralObjectClass %s "
-					"does not match objectClass %s associated "
+					"is subclass of objectClass %s associated "
 					"to entry\n",
-					bsi->bsi_e->e_name.bv_val, soc.bv_val,
+					bsi->bsi_e->e_name.bv_val, soc->soc_cname.bv_val,
 					bsi->bsi_oc->bom_oc->soc_cname.bv_val );
-				backsql_entry_clean( op, bsi->bsi_e );
-				return rc;
 			}
+
+		} else {
+			soc = bsi->bsi_oc->bom_oc;
 		}
 
 		rc = attr_merge_normalize_one( bsi->bsi_e,
 				slap_schema.si_ad_structuralObjectClass,
-				&bsi->bsi_oc->bom_oc->soc_cname,
+				&soc->soc_cname,
 				bsi->bsi_op->o_tmpmemctx );
 		if ( rc != LDAP_SUCCESS ) {
 			backsql_entry_clean( op, bsi->bsi_e );
