@@ -1,5 +1,5 @@
 /* rwm.c - rewrite/remap operations */
-/* $OpenLDAP: pkg/ldap/servers/slapd/overlays/rwm.c,v 1.37.2.17 2007/01/03 08:55:04 hyc Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/overlays/rwm.c,v 1.37.2.20 2007/08/14 09:59:44 ando Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 2003-2007 The OpenLDAP Foundation.
@@ -32,6 +32,7 @@ typedef struct rwm_op_state {
 	struct berval ro_ndn;
 	struct berval r_dn;
 	struct berval r_ndn;
+	AttributeName *mapped_attrs;
 	OpRequest o_request;
 } rwm_op_state;
 
@@ -79,7 +80,7 @@ rwm_op_cleanup( Operation *op, SlapReply *rs )
 			}
 			break;
 		case LDAP_REQ_SEARCH:
-			ch_free( op->ors_attrs );
+			ch_free( ros->mapped_attrs );
 			filter_free_x( op, op->ors_filter );
 			ch_free( op->ors_filterstr.bv_val );
 			op->ors_attrs = ros->ors_attrs;
@@ -121,6 +122,7 @@ rwm_callback_get( Operation *op, SlapReply *rs )
 	return roc;
 }
 
+
 static int
 rwm_op_dn_massage( Operation *op, SlapReply *rs, void *cookie,
 	rwm_op_state *ros )
@@ -769,6 +771,11 @@ rwm_swap_attrs( Operation *op, SlapReply *rs )
 	rwm_op_state *ros = cb->sc_private;
 
 	rs->sr_attrs = ros->ors_attrs;
+
+	/* other overlays might have touched op->ors_attrs, 
+	 * so we restore the original version here, otherwise
+	 * attribute-mapping might fail */
+	op->ors_attrs = ros->mapped_attrs; 
 	
  	return SLAP_CB_CONTINUE;
 }
@@ -843,6 +850,9 @@ rwm_op_search( Operation *op, SlapReply *rs )
 	}
 
 	op->ors_attrs = an;
+	/* store the mapped Attributes for later usage, in
+	 * the case that other overlays change op->ors_attrs */
+	roc->ros.mapped_attrs = an;
 	roc->cb.sc_response = rwm_swap_attrs;
 
 	op->o_callback = &roc->cb;
@@ -1065,6 +1075,7 @@ rwm_attrs( Operation *op, SlapReply *rs, Attribute** a_first, int stripEntryDN )
 	int			rc;
 	Attribute		**ap;
 	int			isupdate;
+	int			check_duplicate_attrs = 0;
 
 	/*
 	 * Rewrite the dn attrs, if needed
@@ -1118,6 +1129,9 @@ rwm_attrs( Operation *op, SlapReply *rs, Attribute** a_first, int stripEntryDN )
 
 			if ( mapping != NULL ) {
 				(*ap)->a_desc = mapping->m_dst_ad;
+
+				/* will need to check for duplicate attrs */
+				check_duplicate_attrs++;
 			}
 		}
 
@@ -1154,6 +1168,7 @@ rwm_attrs( Operation *op, SlapReply *rs, Attribute** a_first, int stripEntryDN )
 
 				rwm_map( &rwmap->rwm_oc, &bv[0], &mapped, RWM_REMAP );
 				if ( BER_BVISNULL( &mapped ) || BER_BVISEMPTY( &mapped ) ) {
+remove_oc:;
 					ch_free( bv[0].bv_val );
 					BER_BVZERO( &bv[0] );
 					if ( &(*ap)->a_vals[last] > &bv[0] ) {
@@ -1164,12 +1179,32 @@ rwm_attrs( Operation *op, SlapReply *rs, Attribute** a_first, int stripEntryDN )
 					bv--;
 
 				} else if ( mapped.bv_val != bv[0].bv_val ) {
+					int	i;
+
+					for ( i = 0; !BER_BVISNULL( &(*ap)->a_vals[ i ] ); i++ ) {
+						if ( &(*ap)->a_vals[ i ] == bv ) {
+							continue;
+						}
+
+						if ( ber_bvstrcasecmp( &mapped, &(*ap)->a_vals[ i ] ) == 0 ) {
+							break;
+						}
+					}
+
+					if ( !BER_BVISNULL( &(*ap)->a_vals[ i ] ) ) {
+						goto remove_oc;
+					}
+
 					/*
 					 * FIXME: after LBER_FREEing
 					 * the value is replaced by
 					 * ch_alloc'ed memory
 					 */
 					ber_bvreplace( &bv[0], &mapped );
+
+					/* FIXME: will need to check
+					 * if the structuralObjectClass
+					 * changed */
 				}
 			}
 
@@ -1223,6 +1258,45 @@ cleanup_attr:;
 		attr_free( a );
 	}
 
+	/* only check if some mapping occurred */
+	if ( check_duplicate_attrs ) {
+		for ( ap = a_first; *ap != NULL; ap = &(*ap)->a_next ) {
+			Attribute	**tap;
+
+			for ( tap = &(*ap)->a_next; *tap != NULL; ) {
+				if ( (*tap)->a_desc == (*ap)->a_desc ) {
+					Entry		e = { 0 };
+					Modification	mod = { 0 };
+					const char	*text = NULL;
+					char		textbuf[ SLAP_TEXT_BUFLEN ];
+					Attribute	*next = (*tap)->a_next;
+
+					BER_BVSTR( &e.e_name, "" );
+					BER_BVSTR( &e.e_nname, "" );
+					e.e_attrs = *ap;
+					mod.sm_op = LDAP_MOD_ADD;
+					mod.sm_desc = (*ap)->a_desc;
+					mod.sm_type = mod.sm_desc->ad_cname;
+					mod.sm_values = (*tap)->a_vals;
+					mod.sm_nvalues = (*tap)->a_nvals;
+
+					(void)modify_add_values( &e, &mod,
+						/* permissive */ 1,
+						&text, textbuf, sizeof( textbuf ) );
+
+					/* should not insert new attrs! */
+					assert( e.e_attrs == *ap );
+
+					attr_free( *tap );
+					*tap = next;
+
+				} else {
+					tap = &(*tap)->a_next;
+				}
+			}
+		}
+	}
+
 	return 0;
 }
 
@@ -1642,7 +1716,6 @@ rwm_db_init(
 	BackendDB	*be )
 {
 	slap_overinst		*on = (slap_overinst *) be->bd_info;
-	struct ldapmapping	*mapping = NULL;
 	struct ldaprwmap	*rwmap;
 #ifdef ENABLE_REWRITE
 	char			*rargv[ 3 ];
@@ -1671,13 +1744,6 @@ rwm_db_init(
 	rewrite_parse( rwmap->rwm_rw, "<suffix massage>", 2, 2, rargv );
 #endif /* ENABLE_REWRITE */
 
-	if ( rwm_map_init( &rwmap->rwm_oc, &mapping ) != LDAP_SUCCESS ||
-			rwm_map_init( &rwmap->rwm_at, &mapping ) != LDAP_SUCCESS )
-	{
-		rc = 1;
-		goto error_return;
-	}
-
 error_return:;
 	on->on_bi.bi_private = (void *)rwmap;
 
